#include "Fit/Fitter.h"
#include "Fit/BinData.h"
#include "Fit/Chi2FCN.h"
#include "TH1.h"
#include "TList.h"
#include "Math/WrappedMultiTF1.h"
#include "HFitInterface.h"
#include "TCanvas.h"
#include "TStyle.h"
 
 
// definition of shared parameter
// background function
int nParB = 10;
int iparB[10] = { 13, // Xi(1820) top        amplitude
                 4, // Xi(1820) top+bottom mean        Shared
                 5,  // Xi(1820) top+bottom sigma       Shared
                 14, // Xi(1690) top          amplitude
                  7, // Xi(1690) top+bottom mean       Shared
                  8, // Xi(1690) top+bottom sigma      Shared
                 15,
                 16,
                 17,
                 18
};
 
// signal + background function
int nParSB = 13;
int iparSB[13] = { 0, // Xi(1530) amplitude
                  1, // Xi(1530) mean
                  2, // Xi(1530) sigma
                  3, // Xi(1820)     bottom amplitude
                  4, // Xi(1820) top+bottom mean       Shared
                  5, // Xi(1820) top+bottom sigma      Shared
                  6, // Xi(1690)     bottom amplitude
                  7, // Xi(1690) top+bottom mean       Shared
                  8, // Xi(1690) top+bottom sigma      Shared
                  9,
                  10,
                  11,
                  12
};

const int Npar = 19;

double gAmplitude1 = 40;
double gCenter1 = 1.53;
double gSigma1 = 0.01;
double gAmplitude2 = 40;
double gAmplitude2top = 200;
double gCenter2 = 1.82;
double gSigma2 = 0.015;

double gAmplitude3 = 40;
double gAmplitude3top = 120;
double gCenter3 = 1.7;
double gSigma3 = 0.015;

double polySB1 = -771.915; 
double polySB2 = 654.205; 
double polySB3 = 225.88; 
double polySB4 = -163.869; 

double polyB1 = -31238.2; 
double polyB2 = 15504.1; 
double polyB3 = 11206.6; 
double polyB4 = -5440.22; 

  double par0[Npar] = { gAmplitude1,gCenter1,gSigma1, //1530 
			gAmplitude2,gCenter2,gSigma2,//1820 
			gAmplitude3,gCenter3,gSigma3,//1690 
			polySB1,polySB2,polySB3,polySB4,
			gAmplitude2top,
			gAmplitude3top,
			polyB1,polyB2,polyB3,polyB4
  };

 
// Create the GlobalCHi2 structure
 
struct GlobalChi2 {
  GlobalChi2(  ROOT::Math::IMultiGenFunction & f1,
	       ROOT::Math::IMultiGenFunction & f2) :
    fChi2_1(&f1), fChi2_2(&f2) {}
 
  double operator() (const double *par) const {
    double p1[nParB];
    for (int i = 0; i < nParB; ++i) p1[i] = par[iparB[i] ];
 
    double p2[nParSB];
    for (int i = 0; i < nParSB; ++i) p2[i] = par[iparSB[i] ];
 
    return (*fChi2_1)(p1) + (*fChi2_2)(p2);
  }
 
  const  ROOT::Math::IMultiGenFunction * fChi2_1;
  const  ROOT::Math::IMultiGenFunction * fChi2_2;
};

void combinedFit() {

  gROOT->Reset();//Reset session memory
  gROOT->SetStyle("Plain");//Make style plain
  gStyle->SetOptStat(0);//Remove the statistics box from histogram

  TFile *hFile1 = new TFile("allKmLam.root");
  TH1D *hB; hFile1->GetObject("CasStar_Kin_MMS_4",hB);

  TFile *hFile2 = new TFile("mAllXi.root");
  TH1D *hSB; hFile2->GetObject("hCasStar_MC_Kin_4",hSB);

  hB->Rebin(8);
  hSB->Rebin(8);

  hB->GetXaxis()->SetRangeUser(1.48,2.0);
  hSB->GetXaxis()->SetRangeUser(1.48,2.0);
 
  TF1 * fB = new TF1("fB","gaus + gaus(3) + pol3(6)",0,100);

 
  // perform now global fit
  TF1 * fSB = new TF1("fSB","gaus + gaus(3) + gaus(6) + pol3(9)",0,100);//!!!!!
 
  ROOT::Math::WrappedMultiTF1 wfB(*fB,1);
  ROOT::Math::WrappedMultiTF1 wfSB(*fSB,1);
 
  ROOT::Fit::DataOptions opt;
  ROOT::Fit::DataRange rangeB;
  // set the data range
  rangeB.SetRange(1.6,2.0);//!!!!!
  ROOT::Fit::BinData dataB(opt,rangeB);
  ROOT::Fit::FillData(dataB, hB);
 
  ROOT::Fit::DataRange rangeSB;
  rangeSB.SetRange(1.48,1.99);//!!!!!
  ROOT::Fit::BinData dataSB(opt,rangeSB);
  ROOT::Fit::FillData(dataSB, hSB);
 
  ROOT::Fit::Chi2Function chi2_B(dataB, wfB);
  ROOT::Fit::Chi2Function chi2_SB(dataSB, wfSB);
 
  GlobalChi2 globalChi2(chi2_B, chi2_SB);
 
  ROOT::Fit::Fitter fitter;

 
  // create before the parameter settings in order to fix or set range on them
  fitter.Config().SetParamsSettings(Npar,par0);
  // fix 5-th parameter
  //fitter.Config().ParSettings(2).Fix();

  //fitter.Config().ParSettings(1).Fix();
  //fitter.Config().ParSettings(2).Fix();

  //fitter.Config().ParSettings(13).Fix();
  //fitter.Config().ParSettings(4).Fix();
  //fitter.Config().ParSettings(5).Fix();

  //fitter.Config().ParSettings(6).Fix();
  //fitter.Config().ParSettings(7).Fix();
  //fitter.Config().ParSettings(8).Fix();

  //fitter.Config().ParSettings(11).Fix();
  //fitter.Config().ParSettings(12).Fix();
  //fitter.Config().ParSettings(13).Fix();
  //fitter.Config().ParSettings(14).Fix();



  // set limits on the third and 4-th parameter
  //fitter.Config().ParSettings(2).SetLimits(-10,-1.E-4);
  fitter.Config().ParSettings(0).SetLimits(0,1000000);
  fitter.Config().ParSettings(0).SetStepSize(5);

  fitter.Config().ParSettings(3).SetLimits(0,1000000);
  fitter.Config().ParSettings(3).SetStepSize(5);
 
  fitter.Config().MinimizerOptions().SetPrintLevel(0);
  fitter.Config().SetMinimizer("Minuit2","Migrad");
 
  // fit FCN function directly
  // (specify optionally data size and flag to indicate that is a chi2 fit)

  fitter.FitFCN(Npar,globalChi2,0,dataB.Size()+dataSB.Size(),true);
  ROOT::Fit::FitResult result = fitter.Result();
  result.Print(std::cout);
 
  TCanvas * c1 = new TCanvas("Simfit","Simultaneous fit of two histograms",
			     10,10,700,700);
  c1->Divide(1,2);
  c1->cd(1);
  gStyle->SetOptFit(1111);
 
  //fB->SetFitResult( result, iparB);
  fB->SetRange(rangeB().first, rangeB().second);
  fB->SetLineColor(kBlue);
  //hB->GetListOfFunctions()->Add(fB);
  hB->Draw("e");
  fB->Draw("same");

  c1->cd(2);
  //fSB->SetFitResult( result, iparSB);
  fSB->SetRange(rangeSB().first, rangeSB().second);
  fSB->SetLineColor(kRed);
  //hSB->GetListOfFunctions()->Add(fSB);
  hSB->Draw("e");
  fSB->Draw("same");
 
 
}

